@@ -4,7 +4,7 @@
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #ifndef __KNET_INTERNALS_H__
@@ -33,7 +33,17 @@
 #define PCKT_FRAG_MAX UINT8_MAX
 #define PCKT_RX_BUFS  512
 
-#define KNET_EPOLL_MAX_EVENTS KNET_DATAFD_MAX
+#define KNET_EPOLL_MAX_EVENTS KNET_DATAFD_MAX + 1
+
+#define KNET_INTERNAL_DATA_CHANNEL KNET_DATAFD_MAX
+
+/*
+ * Size of threads stack. Value is choosen by experimenting, how much is needed
+ * to sucesfully finish test suite, and at the time of writing patch it was
+ * ~300KiB. To have some room for future enhancement it is increased
+ * by factor of 3 and rounded.
+ */
+#define KNET_THREAD_STACK_SIZE (1024 * 1024)
 
 typedef void *knet_transport_link_t; /* per link transport handle */
 typedef void *knet_transport_t;      /* per knet_h transport handle */
@@ -62,7 +72,7 @@ struct knet_link {
 	struct knet_link_status status;
 	/* internals */
 	uint8_t link_id;
-	uint8_t transport_type;                 /* #defined constant from API */
+	uint8_t transport;                      /* #defined constant from API */
 	knet_transport_link_t transport_link;   /* link_info_t from transport */
 	int outsock;
 	unsigned int configured:1;		/* set to 1 if src/dst have been configured transport initialized on this link*/
@@ -71,13 +81,16 @@ struct knet_link {
 	uint8_t received_pong;
 	struct timespec ping_last;
 	/* used by PMTUD thread as temp per-link variables and should always contain the onwire_len value! */
-	uint32_t proto_overhead;
+	uint32_t proto_overhead;		/* IP + UDP/SCTP overhead. NOT to be confused
+						   with stats.proto_overhead that includes also knet headers
+						   and crypto headers */
 	struct timespec pmtud_last;
 	uint32_t last_ping_size;
 	uint32_t last_good_mtu;
 	uint32_t last_bad_mtu;
 	uint32_t last_sent_mtu;
 	uint32_t last_recv_mtu;
+	uint32_t pmtud_crypto_timeout_multiplier;/* used by PMTUd to adjust timeouts on high loads */
 	uint8_t has_valid_mtu;
 };
 
@@ -90,8 +103,8 @@ struct knet_host_defrag_buf {
 	uint8_t frag_recv;		/* how many frags did we receive */
 	uint8_t frag_map[PCKT_FRAG_MAX];/* bitmap of what we received? */
 	uint8_t	last_first;		/* special case if we receive the last fragment first */
-	uint16_t frag_size;		/* normal frag size (not the last one) */
-	uint16_t last_frag_size;	/* the last fragment might not be aligned with MTU size */
+	ssize_t frag_size;		/* normal frag size (not the last one) */
+	ssize_t last_frag_size;		/* the last fragment might not be aligned with MTU size */
 	struct timespec last_update;	/* keep time of the last pckt */
 };
 
@@ -130,10 +143,11 @@ struct knet_sock {
 };
 
 struct knet_fd_trackers {
-	uint8_t transport; /* transport type (UDP/SCTP...) */
-	uint8_t data_type; /* internal use for transport to define what data are associated
-			    * to this fd */
-	void *data;	   /* pointer to the data */
+	uint8_t transport;		    /* transport type (UDP/SCTP...) */
+	uint8_t data_type;		    /* internal use for transport to define what data are associated
+					     * with this fd */
+	void *data;			    /* pointer to the data */
+	void *access_list_match_entry_head; /* pointer to access list match_entry list head */
 };
 
 #define KNET_MAX_FDS KNET_MAX_HOST * KNET_MAX_LINK * 4
@@ -150,7 +164,7 @@ struct knet_handle_stats_extra {
 struct knet_handle {
 	knet_node_id_t host_id;
 	unsigned int enabled:1;
-	struct knet_sock sockfd[KNET_DATAFD_MAX];
+	struct knet_sock sockfd[KNET_DATAFD_MAX + 1];
 	int logfd;
 	uint8_t log_levels[KNET_MAX_SUBSYSTEMS];
 	int hostsockfd[2];
@@ -158,7 +172,9 @@ struct knet_handle {
 	int send_to_links_epollfd;
 	int recv_from_links_epollfd;
 	int dst_link_handler_epollfd;
+	uint8_t use_access_lists; /* set to 0 for disable, 1 for enable */
 	unsigned int pmtud_interval;
+	unsigned int manual_mtu;
 	unsigned int data_mtu;	/* contains the max data size that we can send onwire
 				 * without frags */
 	struct knet_host *host_head;
@@ -176,6 +192,7 @@ struct knet_handle {
 	struct knet_header *pingbuf;
 	struct knet_header *pmtudbuf;
 	uint8_t threads_status[KNET_THREAD_MAX];
+	uint8_t threads_flush_queue[KNET_THREAD_MAX];
 	pthread_mutex_t threads_status_mutex;
 	pthread_t send_to_links_thread;
 	pthread_t recv_from_links_thread;
@@ -195,7 +212,6 @@ struct knet_handle {
 	int pmtud_forcerun;
 	int pmtud_abort;
 	struct crypto_instance *crypto_instance;
-	size_t sec_header_size;
 	size_t sec_block_size;
 	size_t sec_hash_size;
 	size_t sec_salt_size;
@@ -255,6 +271,32 @@ extern pthread_rwlock_t shlib_rwlock;       /* global shared lib load lock */
  *       for every protocol.
  */
 
+/*
+ * for now knet supports only IP protocols (udp/sctp)
+ * in future there might be others like ARP
+ * or TIPC.
+ * keep this around as transport information
+ * to use for access lists and other operations
+ */
+
+#define TRANSPORT_PROTO_LOOPBACK 0
+#define TRANSPORT_PROTO_IP_PROTO 1
+
+/*
+ * some transports like SCTP can filter incoming
+ * connections before knet has to process
+ * any packets.
+ * GENERIC_ACL -> packet has to be read and filterted
+ * PROTO_ACL -> transport provides filtering at lower levels
+ *              and packet does not need to be processed
+ */
+
+typedef enum {
+	USE_NO_ACL,
+	USE_GENERIC_ACL,
+	USE_PROTO_ACL
+} transport_acl;
+
 /*
  * make it easier to map values in transports.c
  */
@@ -269,6 +311,9 @@ typedef struct knet_transport_ops {
 	const uint8_t transport_id;
 	const uint8_t built_in;
 
+	uint8_t transport_protocol;
+	transport_acl transport_acl_type;
+
 /*
  * connection oriented protocols like SCTP
  * don´t need dst_addr in sendto calls and
@@ -307,6 +352,12 @@ typedef struct knet_transport_ops {
  */
 	int (*transport_link_dyn_connect)(knet_handle_t knet_h, int sockfd, struct knet_link *link);
 
+
+/*
+ * return the fd to use for access lists
+ */
+	int (*transport_link_get_acl_fd)(knet_handle_t knet_h, struct knet_link *link);
+
 /*
  * per transport error handling of recvmmsg
  * (see _handle_recv_from_links comments for details)
