@@ -4,7 +4,7 @@
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -20,6 +20,7 @@
 #include "transports.h"
 #include "host.h"
 #include "threads_common.h"
+#include "links_acl.h"
 
 int _link_updown(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 		 unsigned int enabled, unsigned int connected)
@@ -42,13 +43,13 @@ int _link_updown(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 	if (connected) {
 		time(&link->status.stats.last_up_times[link->status.stats.last_up_time_index]);
 		link->status.stats.up_count++;
-		if (++link->status.stats.last_up_time_index > MAX_LINK_EVENTS) {
+		if (++link->status.stats.last_up_time_index >= MAX_LINK_EVENTS) {
 			link->status.stats.last_up_time_index = 0;
 		}
 	} else {
 		time(&link->status.stats.last_down_times[link->status.stats.last_down_time_index]);
 		link->status.stats.down_count++;
-		if (++link->status.stats.last_down_time_index > MAX_LINK_EVENTS) {
+		if (++link->status.stats.last_down_time_index >= MAX_LINK_EVENTS) {
 			link->status.stats.last_down_time_index = 0;
 		}
 	}
@@ -218,6 +219,7 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		}
 	}
 
+	link->pmtud_crypto_timeout_multiplier = KNET_LINK_PMTUD_CRYPTO_TIMEOUT_MULTIPLIER_MIN;
 	link->pong_count = KNET_LINK_DEFAULT_PONG_COUNT;
 	link->has_valid_mtu = 0;
 	link->ping_interval = KNET_LINK_DEFAULT_PING_INTERVAL * 1000; /* microseconds */
@@ -234,6 +236,26 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		err = -1;
 		goto exit_unlock;
 	}
+
+	/*
+	 * we can only configure default access lists if we know both endpoints
+	 * and the protocol uses GENERIC_ACL, otherwise the protocol has
+	 * to setup their own access lists above in transport_link_set_config.
+	 */
+	if ((transport_get_acl_type(knet_h, transport) == USE_GENERIC_ACL) &&
+	    (link->dynamic == KNET_LINK_STATIC)) {
+		log_debug(knet_h, KNET_SUB_LINK, "Configuring default access lists for host: %u link: %u socket: %d",
+			  host_id, link_id, link->outsock);
+		if ((check_add(knet_h, link->outsock, transport, -1,
+			       &link->dst_addr, &link->dst_addr,
+			       CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != EEXIST)) {
+			log_warn(knet_h, KNET_SUB_LINK, "Failed to configure default access lists for host: %u link: %u", host_id, link_id);
+			savederrno = errno;
+			err = -1;
+			goto exit_unlock;
+		}
+	}
+
 	link->configured = 1;
 	log_debug(knet_h, KNET_SUB_LINK, "host: %u link: %u is configured",
 		  host_id, link_id);
@@ -244,7 +266,32 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		host->status.reachable = 1;
 		link->status.mtu = KNET_PMTUD_SIZE_V6;
 	} else {
-		link->status.mtu =  KNET_PMTUD_MIN_MTU_V4 - KNET_HEADER_ALL_SIZE - knet_h->sec_header_size;
+		/*
+		 * calculate the minimum MTU that is safe to use,
+		 * based on RFCs and that each network device should
+		 * be able to support without any troubles
+		 */
+		if (link->dynamic == KNET_LINK_STATIC) {
+			/*
+			 * with static link we can be more precise than using
+			 * the generic calc_min_mtu()
+			 */
+			switch (link->dst_addr.ss_family) {
+				case AF_INET6:
+					link->status.mtu =  calc_max_data_outlen(knet_h, KNET_PMTUD_MIN_MTU_V6 - (KNET_PMTUD_OVERHEAD_V6 + link->proto_overhead));
+					break;
+				case AF_INET:
+					link->status.mtu =  calc_max_data_outlen(knet_h, KNET_PMTUD_MIN_MTU_V4 - (KNET_PMTUD_OVERHEAD_V4 + link->proto_overhead));
+					break;
+			}
+		} else {
+			/*
+			 * for dynamic links we start with the minimum MTU
+			 * possible and PMTUd will kick in immediately
+			 * after connection status is 1
+			 */
+			link->status.mtu =  calc_min_mtu(knet_h);
+		}
 		link->has_valid_mtu = 1;
 	}
 
@@ -330,7 +377,7 @@ int knet_link_get_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 
 	memmove(src_addr, &link->src_addr, sizeof(struct sockaddr_storage));
 
-	*transport = link->transport_type;
+	*transport = link->transport;
 	*flags = link->flags;
 
 	if (link->dynamic == KNET_LINK_STATIC) {
@@ -351,6 +398,8 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	int savederrno = 0, err = 0;
 	struct knet_host *host;
 	struct knet_link *link;
+	int sock;
+	uint8_t transport;
 
 	if (!knet_h) {
 		errno = EINVAL;
@@ -397,6 +446,32 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 		goto exit_unlock;
 	}
 
+	/*
+	 * remove well known access lists here.
+	 * After the transport has done clearing the config,
+	 * then we can remove any leftover access lists if the link
+	 * is no longer in use.
+	 */
+	if ((transport_get_acl_type(knet_h, link->transport) == USE_GENERIC_ACL) &&
+	    (link->dynamic == KNET_LINK_STATIC)) {
+		if ((check_rm(knet_h, link->outsock, link->transport,
+			      &link->dst_addr, &link->dst_addr,
+			      CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != ENOENT)) {
+			err = -1;
+			savederrno = errno;
+			log_err(knet_h, KNET_SUB_LINK, "Host %u link %u: unable to remove default access list",
+				host_id, link_id);
+			goto exit_unlock;
+		}
+	}
+
+	/*
+	 * cache it for later as we don't know if the transport
+	 * will clear link info during clear_config.
+	 */
+	sock = link->outsock;
+	transport = link->transport;
+
 	if ((transport_link_clear_config(knet_h, link) < 0)  &&
 	    (errno != EBUSY)) {
 		savederrno = errno;
@@ -404,6 +479,15 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 		goto exit_unlock;
 	}
 
+	/*
+	 * remove any other access lists when the socket is no
+	 * longer in use by the transport.
+	 */
+	if ((transport_get_acl_type(knet_h, link->transport) == USE_GENERIC_ACL) &&
+	    (knet_h->knet_transport_fd_tracker[sock].transport == KNET_MAX_TRANSPORTS)) {
+		check_rmall(knet_h, sock, transport);
+	}
+
 	memset(link, 0, sizeof(struct knet_link));
 	link->link_id = link_id;
 
@@ -1090,3 +1174,346 @@ exit_unlock:
 	errno = err ? savederrno : 0;
 	return err;
 }
+
+int knet_link_add_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+		      struct sockaddr_storage *ss1,
+		      struct sockaddr_storage *ss2,
+		      check_type_t type, check_acceptreject_t acceptreject)
+{
+	int savederrno = 0, err = 0;
+	struct knet_host *host;
+	struct knet_link *link;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (!ss1) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) &&
+	    (type != CHECK_TYPE_MASK) &&
+	    (type != CHECK_TYPE_RANGE)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((acceptreject != CHECK_ACCEPT) &&
+	    (acceptreject != CHECK_REJECT)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) && (!ss2)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type == CHECK_TYPE_RANGE) &&
+	    (ss1->ss_family != ss2->ss_family)) {
+			errno = EINVAL;
+			return -1;
+	}
+
+	if (link_id >= KNET_MAX_LINK) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = get_global_wrlock(knet_h);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to get write lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	host = knet_h->host_index[host_id];
+	if (!host) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "Unable to find host %u: %s",
+			host_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	link = &host->link[link_id];
+
+	if (!link->configured) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is not configured: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	if (link->dynamic != KNET_LINK_DYNIP) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is a point to point connection: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	err = check_add(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport, -1,
+			ss1, ss2, type, acceptreject);
+	savederrno = errno;
+
+exit_unlock:
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	errno = savederrno;
+	return err;
+}
+
+int knet_link_insert_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+			 int index,
+			 struct sockaddr_storage *ss1,
+			 struct sockaddr_storage *ss2,
+			 check_type_t type, check_acceptreject_t acceptreject)
+{
+	int savederrno = 0, err = 0;
+	struct knet_host *host;
+	struct knet_link *link;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (!ss1) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) &&
+	    (type != CHECK_TYPE_MASK) &&
+	    (type != CHECK_TYPE_RANGE)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((acceptreject != CHECK_ACCEPT) &&
+	    (acceptreject != CHECK_REJECT)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) && (!ss2)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type == CHECK_TYPE_RANGE) &&
+	    (ss1->ss_family != ss2->ss_family)) {
+			errno = EINVAL;
+			return -1;
+	}
+
+	if (link_id >= KNET_MAX_LINK) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = get_global_wrlock(knet_h);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to get write lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	host = knet_h->host_index[host_id];
+	if (!host) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "Unable to find host %u: %s",
+			host_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	link = &host->link[link_id];
+
+	if (!link->configured) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is not configured: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	if (link->dynamic != KNET_LINK_DYNIP) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is a point to point connection: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	err = check_add(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport, index,
+			ss1, ss2, type, acceptreject);
+	savederrno = errno;
+
+exit_unlock:
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	errno = savederrno;
+	return err;
+}
+
+int knet_link_rm_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+		     struct sockaddr_storage *ss1,
+		     struct sockaddr_storage *ss2,
+		     check_type_t type, check_acceptreject_t acceptreject)
+{
+	int savederrno = 0, err = 0;
+	struct knet_host *host;
+	struct knet_link *link;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (!ss1) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) &&
+	    (type != CHECK_TYPE_MASK) &&
+	    (type != CHECK_TYPE_RANGE)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((acceptreject != CHECK_ACCEPT) &&
+	    (acceptreject != CHECK_REJECT)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type != CHECK_TYPE_ADDRESS) && (!ss2)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if ((type == CHECK_TYPE_RANGE) &&
+	    (ss1->ss_family != ss2->ss_family)) {
+			errno = EINVAL;
+			return -1;
+	}
+
+	if (link_id >= KNET_MAX_LINK) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = get_global_wrlock(knet_h);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to get write lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	host = knet_h->host_index[host_id];
+	if (!host) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "Unable to find host %u: %s",
+			host_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	link = &host->link[link_id];
+
+	if (!link->configured) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is not configured: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	if (link->dynamic != KNET_LINK_DYNIP) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is a point to point connection: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	err = check_rm(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport,
+		       ss1, ss2, type, acceptreject);
+	savederrno = errno;
+
+exit_unlock:
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	errno = savederrno;
+	return err;
+}
+
+int knet_link_clear_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id)
+{
+	int savederrno = 0, err = 0;
+	struct knet_host *host;
+	struct knet_link *link;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (link_id >= KNET_MAX_LINK) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = get_global_wrlock(knet_h);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to get write lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	host = knet_h->host_index[host_id];
+	if (!host) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "Unable to find host %u: %s",
+			host_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	link = &host->link[link_id];
+
+	if (!link->configured) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is not configured: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	if (link->dynamic != KNET_LINK_DYNIP) {
+		err = -1;
+		savederrno = EINVAL;
+		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is a point to point connection: %s",
+			host_id, link_id, strerror(savederrno));
+		goto exit_unlock;
+	}
+
+	check_rmall(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport);
+
+exit_unlock:
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	errno = savederrno;
+	return err;
+}
