@@ -3,7 +3,7 @@
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under GPL-2.0+
  */
 
 #include "config.h"
@@ -293,7 +293,7 @@ static void test_sctp(void)
 	}
 
 	printf("Wait 2 seconds for sockets to connect\n");
-	sleep(2);
+	test_sleep(knet_h, 2);
 
 	printf("Test knet_link_set_enable with correct values (0)\n");
 
