@@ -3,7 +3,7 @@
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under GPL-2.0+
  */
 
 #ifndef __KNET_TEST_COMMON_H__
@@ -70,6 +70,7 @@ int stop_logthread(void);
 int make_local_sockaddr(struct sockaddr_storage *lo, uint16_t offset);
 int make_local_sockaddr6(struct sockaddr_storage *lo, uint16_t offset);
 int wait_for_host(knet_handle_t knet_h, uint16_t host_id, int seconds, int logfd, FILE *std);
-int wait_for_packet(knet_handle_t knet_h, int seconds, int datafd);
+int wait_for_packet(knet_handle_t knet_h, int seconds, int datafd, int logfd, FILE *std);
+void test_sleep(knet_handle_t knet_h, int seconds);
 
 #endif
