@@ -4,7 +4,7 @@
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -22,44 +22,62 @@
 #include "threads_common.h"
 #include "threads_pmtud.h"
 
+static int _calculate_manual_mtu(knet_handle_t knet_h, struct knet_link *dst_link)
+{
+	size_t ipproto_overhead_len;	/* onwire packet overhead (protocol based) */
+
+	switch (dst_link->dst_addr.ss_family) {
+		case AF_INET6:
+			ipproto_overhead_len = KNET_PMTUD_OVERHEAD_V6 + dst_link->proto_overhead;
+			break;
+		case AF_INET:
+			ipproto_overhead_len = KNET_PMTUD_OVERHEAD_V4 + dst_link->proto_overhead;
+			break;
+		default:
+			log_debug(knet_h, KNET_SUB_PMTUD, "unknown protocol");
+			return 0;
+			break;
+	}
+
+	dst_link->status.mtu = calc_max_data_outlen(knet_h, knet_h->manual_mtu - ipproto_overhead_len);
+
+	return 1;
+}
+
 static int _handle_check_link_pmtud(knet_handle_t knet_h, struct knet_host *dst_host, struct knet_link *dst_link)
 {
 	int err, ret, savederrno, mutex_retry_limit, failsafe, use_kernel_mtu, warn_once;
-	uint32_t kernel_mtu; /* record kernel_mtu from EMSGSIZE */
-	size_t onwire_len;   /* current packet onwire size */
-	size_t overhead_len; /* onwire packet overhead (protocol based) */
-	size_t max_mtu_len;  /* max mtu for protocol */
-	size_t data_len;     /* how much data we can send in the packet
-			      * generally would be onwire_len - overhead_len
-			      * needs to be adjusted for crypto
-			      */
-	size_t pad_len;	     /* crypto packet pad size, needs to move into crypto.c callbacks */
-	ssize_t len;	     /* len of what we were able to sendto onwire */
+	uint32_t kernel_mtu;		/* record kernel_mtu from EMSGSIZE */
+	size_t onwire_len;   		/* current packet onwire size */
+	size_t ipproto_overhead_len;	/* onwire packet overhead (protocol based) */
+	size_t max_mtu_len;		/* max mtu for protocol */
+	size_t data_len;		/* how much data we can send in the packet
+					 * generally would be onwire_len - ipproto_overhead_len
+					 * needs to be adjusted for crypto
+					 */
+	size_t app_mtu_len;		/* real data that we can send onwire */
+	ssize_t len;			/* len of what we were able to sendto onwire */
 
-	struct timespec ts;
-	unsigned long long pong_timeout_adj_tmp;
+	struct timespec ts, pmtud_crypto_start_ts, pmtud_crypto_stop_ts;
+	unsigned long long pong_timeout_adj_tmp, timediff;
+	int pmtud_crypto_reduce = 1;
 	unsigned char *outbuf = (unsigned char *)knet_h->pmtudbuf;
 
 	warn_once = 0;
 
 	mutex_retry_limit = 0;
 	failsafe = 0;
-	pad_len = 0;
-
-	dst_link->last_bad_mtu = 0;
 
 	knet_h->pmtudbuf->khp_pmtud_link = dst_link->link_id;
 
 	switch (dst_link->dst_addr.ss_family) {
 		case AF_INET6:
 			max_mtu_len = KNET_PMTUD_SIZE_V6;
-			overhead_len = KNET_PMTUD_OVERHEAD_V6 + dst_link->proto_overhead;
-			dst_link->last_good_mtu = dst_link->last_ping_size + overhead_len;
+			ipproto_overhead_len = KNET_PMTUD_OVERHEAD_V6 + dst_link->proto_overhead;
 			break;
 		case AF_INET:
 			max_mtu_len = KNET_PMTUD_SIZE_V4;
-			overhead_len = KNET_PMTUD_OVERHEAD_V4 + dst_link->proto_overhead;
-			dst_link->last_good_mtu = dst_link->last_ping_size + overhead_len;
+			ipproto_overhead_len = KNET_PMTUD_OVERHEAD_V4 + dst_link->proto_overhead;
 			break;
 		default:
 			log_debug(knet_h, KNET_SUB_PMTUD, "PMTUD aborted, unknown protocol");
@@ -67,6 +85,9 @@ static int _handle_check_link_pmtud(knet_handle_t knet_h, struct knet_host *dst_
 			break;
 	}
 
+	dst_link->last_bad_mtu = 0;
+	dst_link->last_good_mtu = dst_link->last_ping_size + ipproto_overhead_len;
+
 	/*
 	 * discovery starts from the top because kernel will
 	 * refuse to send packets > current iface mtu.
@@ -92,43 +113,40 @@ restart:
 		failsafe++;
 	}
 
-	data_len = onwire_len - overhead_len;
+	/*
+	 * common to all packets
+	 */
+
+	/*
+	 * calculate the application MTU based on current onwire_len minus ipproto_overhead_len
+	 */
+
+	app_mtu_len = calc_max_data_outlen(knet_h, onwire_len - ipproto_overhead_len);
+
+	/*
+	 * recalculate onwire len back that might be different based
+	 * on data padding from crypto layer.
+	 */
+
+	onwire_len = calc_data_outlen(knet_h, app_mtu_len + KNET_HEADER_ALL_SIZE) + ipproto_overhead_len;
+
+	/*
+	 * calculate the size of what we need to send to sendto(2).
+	 * see also onwire.c for packet format explanation.
+	 */
+	data_len = app_mtu_len + knet_h->sec_hash_size + knet_h->sec_salt_size + KNET_HEADER_ALL_SIZE;
 
 	if (knet_h->crypto_instance) {
-
-		if (knet_h->sec_block_size) {
-			pad_len = knet_h->sec_block_size - (data_len % knet_h->sec_block_size);
-			if (pad_len == knet_h->sec_block_size) {
-				pad_len = 0;
-			}
-			data_len = data_len + pad_len;
-		}
-
-		data_len = data_len + (knet_h->sec_hash_size + knet_h->sec_salt_size + knet_h->sec_block_size);
-
-		if (knet_h->sec_block_size) {
-			while (data_len + overhead_len >= max_mtu_len) {
-				data_len = data_len - knet_h->sec_block_size;
-			}
-		}
-
-		if (dst_link->last_bad_mtu) {
-			while (data_len + overhead_len >= dst_link->last_bad_mtu) {
-				data_len = data_len - (knet_h->sec_hash_size + knet_h->sec_salt_size + knet_h->sec_block_size);
-			}
-		}
-
-		if (data_len < (knet_h->sec_hash_size + knet_h->sec_salt_size + knet_h->sec_block_size) + 1) {
+		if (data_len < (knet_h->sec_hash_size + knet_h->sec_salt_size) + 1) {
 			log_debug(knet_h, KNET_SUB_PMTUD, "Aborting PMTUD process: link mtu smaller than crypto header detected (link might have been disconnected)");
 			return -1;
 		}
 
-		onwire_len = data_len + overhead_len;
 		knet_h->pmtudbuf->khp_pmtud_size = onwire_len;
 
 		if (crypto_encrypt_and_sign(knet_h,
 					    (const unsigned char *)knet_h->pmtudbuf,
-					    data_len - (knet_h->sec_hash_size + knet_h->sec_salt_size + knet_h->sec_block_size),
+					    data_len - (knet_h->sec_hash_size + knet_h->sec_salt_size),
 					    knet_h->pmtudbuf_crypt,
 					    (ssize_t *)&data_len) < 0) {
 			log_debug(knet_h, KNET_SUB_PMTUD, "Unable to crypto pmtud packet");
@@ -137,11 +155,8 @@ restart:
 
 		outbuf = knet_h->pmtudbuf_crypt;
 		knet_h->stats_extra.tx_crypt_pmtu_packets++;
-
 	} else {
-
 		knet_h->pmtudbuf->khp_pmtud_size = onwire_len;
-
 	}
 
 	/* link has gone down, aborting pmtud */
@@ -172,7 +187,7 @@ restart:
 		return -1;
 	}
 retry:
-	if (transport_get_connection_oriented(knet_h, dst_link->transport_type) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
+	if (transport_get_connection_oriented(knet_h, dst_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
 		len = sendto(dst_link->outsock, outbuf, data_len, MSG_DONTWAIT | MSG_NOSIGNAL,
 			     (struct sockaddr *) &dst_link->dst_addr, sizeof(struct sockaddr_storage));
 	} else {
@@ -196,7 +211,7 @@ retry:
 
 	kernel_mtu = 0;
 
-	err = transport_tx_sock_error(knet_h, dst_link->transport_type, dst_link->outsock, len, savederrno);
+	err = transport_tx_sock_error(knet_h, dst_link->transport, dst_link->outsock, len, savederrno);
 	switch(err) {
 		case -1: /* unrecoverable error */
 			log_debug(knet_h, KNET_SUB_PMTUD, "Unable to send pmtu packet (sendto): %d %s", savederrno, strerror(savederrno));
@@ -250,6 +265,15 @@ retry:
 			return -1;
 		}
 
+		/*
+		 * non fatal, we can wait the next round to reduce the
+		 * multiplier
+		 */
+		if (clock_gettime(CLOCK_MONOTONIC, &pmtud_crypto_start_ts) < 0) {
+			log_debug(knet_h, KNET_SUB_PMTUD, "Unable to get current time: %s", strerror(errno));
+			pmtud_crypto_reduce = 0;
+		}
+
 		/*
 		 * set PMTUd reply timeout to match pong_timeout on a given link
 		 *
@@ -269,7 +293,7 @@ retry:
 			/*
 			 * crypto, under pressure, is a royal PITA
 			 */
-			pong_timeout_adj_tmp = dst_link->pong_timeout_adj * 2;
+			pong_timeout_adj_tmp = dst_link->pong_timeout_adj * dst_link->pmtud_crypto_timeout_multiplier;
 		} else {
 			pong_timeout_adj_tmp = dst_link->pong_timeout_adj;
 		}
@@ -295,7 +319,11 @@ retry:
 			return -1;
 		}
 
-		if (shutdown_in_progress(knet_h)) {
+		/*
+		 * we cannot use shutdown_in_progress in here because
+		 * we already hold the read lock
+		 */
+		if (knet_h->fini_in_progress) {
 			pthread_mutex_unlock(&knet_h->pmtud_mutex);
 			log_debug(knet_h, KNET_SUB_PMTUD, "PMTUD aborted. shutdown in progress");
 			return -1;
@@ -303,6 +331,17 @@ retry:
 
 		if (ret) {
 			if (ret == ETIMEDOUT) {
+				if ((knet_h->crypto_instance) && (dst_link->pmtud_crypto_timeout_multiplier < KNET_LINK_PMTUD_CRYPTO_TIMEOUT_MULTIPLIER_MAX)) {
+					dst_link->pmtud_crypto_timeout_multiplier = dst_link->pmtud_crypto_timeout_multiplier * 2;
+					pmtud_crypto_reduce = 0;
+					log_debug(knet_h, KNET_SUB_PMTUD,
+							"Increasing PMTUd response timeout multiplier to (%u) for host %u link: %u",
+							dst_link->pmtud_crypto_timeout_multiplier,
+							dst_host->host_id,
+							dst_link->link_id);
+					pthread_mutex_unlock(&knet_h->pmtud_mutex);
+					goto restart;
+				}
 				if (!warn_once) {
 					log_warn(knet_h, KNET_SUB_PMTUD,
 							"possible MTU misconfiguration detected. "
@@ -331,6 +370,23 @@ retry:
 			}
 		}
 
+		if ((knet_h->crypto_instance) && (pmtud_crypto_reduce == 1) &&
+		    (dst_link->pmtud_crypto_timeout_multiplier > KNET_LINK_PMTUD_CRYPTO_TIMEOUT_MULTIPLIER_MIN)) {
+			if (!clock_gettime(CLOCK_MONOTONIC, &pmtud_crypto_stop_ts)) {
+				timespec_diff(pmtud_crypto_start_ts, pmtud_crypto_stop_ts, &timediff);
+				if (((pong_timeout_adj_tmp * 1000) / 2) > timediff) {
+					dst_link->pmtud_crypto_timeout_multiplier = dst_link->pmtud_crypto_timeout_multiplier / 2;
+					log_debug(knet_h, KNET_SUB_PMTUD,
+							"Decreasing PMTUd response timeout multiplier to (%u) for host %u link: %u",
+							dst_link->pmtud_crypto_timeout_multiplier,
+							dst_host->host_id,
+							dst_link->link_id);
+				}
+			} else {
+				log_debug(knet_h, KNET_SUB_PMTUD, "Unable to get current time: %s", strerror(errno));
+			}
+		}
+
 		if ((dst_link->last_recv_mtu != onwire_len) || (ret)) {
 			dst_link->last_bad_mtu = onwire_len;
 		} else {
@@ -353,7 +409,7 @@ retry:
 				/*
 				 * account for IP overhead, knet headers and crypto in PMTU calculation
 				 */
-				dst_link->status.mtu = onwire_len - dst_link->status.proto_overhead;
+				dst_link->status.mtu = calc_max_data_outlen(knet_h, onwire_len - ipproto_overhead_len);
 				pthread_mutex_unlock(&knet_h->pmtud_mutex);
 				return 0;
 			}
@@ -373,35 +429,40 @@ retry:
 	goto restart;
 }
 
-static int _handle_check_pmtud(knet_handle_t knet_h, struct knet_host *dst_host, struct knet_link *dst_link, unsigned int *min_mtu, int force_run)
+static int _handle_check_pmtud(knet_handle_t knet_h, struct knet_host *dst_host, struct knet_link *dst_link, int force_run)
 {
 	uint8_t saved_valid_pmtud;
 	unsigned int saved_pmtud;
 	struct timespec clock_now;
 	unsigned long long diff_pmtud, interval;
 
+	if (clock_gettime(CLOCK_MONOTONIC, &clock_now) != 0) {
+		log_debug(knet_h, KNET_SUB_PMTUD, "Unable to get monotonic clock");
+		return 0;
+	}
+
 	if (!force_run) {
 		interval = knet_h->pmtud_interval * 1000000000llu; /* nanoseconds */
 
-		if (clock_gettime(CLOCK_MONOTONIC, &clock_now) != 0) {
-			log_debug(knet_h, KNET_SUB_PMTUD, "Unable to get monotonic clock");
-			return 0;
-		}
-
 		timespec_diff(dst_link->pmtud_last, clock_now, &diff_pmtud);
 
 		if (diff_pmtud < interval) {
-			*min_mtu = dst_link->status.mtu;
 			return dst_link->has_valid_mtu;
 		}
 	}
 
+	/*
+	 * status.proto_overhead should include all IP/(UDP|SCTP)/knet headers
+	 *
+	 * please note that it is not the same as link->proto_overhead that
+	 * includes only either UDP or SCTP (at the moment) overhead.
+	 */
 	switch (dst_link->dst_addr.ss_family) {
 		case AF_INET6:
-			dst_link->status.proto_overhead = KNET_PMTUD_OVERHEAD_V6 + dst_link->proto_overhead + KNET_HEADER_ALL_SIZE + knet_h->sec_header_size;
+			dst_link->status.proto_overhead = KNET_PMTUD_OVERHEAD_V6 + dst_link->proto_overhead + KNET_HEADER_ALL_SIZE + knet_h->sec_hash_size + knet_h->sec_salt_size;
 			break;
 		case AF_INET:
-			dst_link->status.proto_overhead = KNET_PMTUD_OVERHEAD_V4 + dst_link->proto_overhead + KNET_HEADER_ALL_SIZE + knet_h->sec_header_size;
+			dst_link->status.proto_overhead = KNET_PMTUD_OVERHEAD_V4 + dst_link->proto_overhead + KNET_HEADER_ALL_SIZE + knet_h->sec_hash_size + knet_h->sec_salt_size;
 			break;
 	}
 
@@ -421,26 +482,13 @@ static int _handle_check_pmtud(knet_handle_t knet_h, struct knet_host *dst_host,
 		}
 		dst_link->has_valid_mtu = 0;
 	} else {
-		dst_link->has_valid_mtu = 1;
-		switch (dst_link->dst_addr.ss_family) {
-			case AF_INET6:
-				if (((dst_link->status.mtu + dst_link->status.proto_overhead) < KNET_PMTUD_MIN_MTU_V6) ||
-				    ((dst_link->status.mtu + dst_link->status.proto_overhead) > KNET_PMTUD_SIZE_V6)) {
-					log_debug(knet_h, KNET_SUB_PMTUD,
-						  "PMTUD detected an IPv6 MTU out of bound value (%u) for host: %u link: %u.",
-						  dst_link->status.mtu + dst_link->status.proto_overhead, dst_host->host_id, dst_link->link_id);
-					dst_link->has_valid_mtu = 0;
-				}
-				break;
-			case AF_INET:
-				if (((dst_link->status.mtu + dst_link->status.proto_overhead) < KNET_PMTUD_MIN_MTU_V4) ||
-				    ((dst_link->status.mtu + dst_link->status.proto_overhead) > KNET_PMTUD_SIZE_V4)) {
-					log_debug(knet_h, KNET_SUB_PMTUD,
-						  "PMTUD detected an IPv4 MTU out of bound value (%u) for host: %u link: %u.",
-						  dst_link->status.mtu + dst_link->status.proto_overhead, dst_host->host_id, dst_link->link_id);
-					dst_link->has_valid_mtu = 0;
-				}
-				break;
+		if (dst_link->status.mtu < calc_min_mtu(knet_h)) {
+			log_info(knet_h, KNET_SUB_PMTUD,
+				 "Invalid MTU detected for host: %u link: %u mtu: %u",
+				 dst_host->host_id, dst_link->link_id, dst_link->status.mtu);
+			dst_link->has_valid_mtu = 0;
+		} else {
+			dst_link->has_valid_mtu = 1;
 		}
 		if (dst_link->has_valid_mtu) {
 			if ((saved_pmtud) && (saved_pmtud != dst_link->status.mtu)) {
@@ -449,9 +497,6 @@ static int _handle_check_pmtud(knet_handle_t knet_h, struct knet_host *dst_host,
 			}
 			log_debug(knet_h, KNET_SUB_PMTUD, "PMTUD completed for host: %u link: %u current link mtu: %u",
 				  dst_host->host_id, dst_link->link_id, dst_link->status.mtu);
-			if (dst_link->status.mtu < *min_mtu) {
-				*min_mtu = dst_link->status.mtu;
-			}
 
 			/*
 			 * set pmtud_last, if we can, after we are done with the PMTUd process
@@ -465,7 +510,7 @@ static int _handle_check_pmtud(knet_handle_t knet_h, struct knet_host *dst_host,
 	}
 
 	if (saved_valid_pmtud != dst_link->has_valid_mtu) {
-		_host_dstcache_update_sync(knet_h, dst_host);
+		_host_dstcache_update_async(knet_h, dst_host);
 	}
 
 	return dst_link->has_valid_mtu;
@@ -477,14 +522,14 @@ void *_handle_pmtud_link_thread(void *data)
 	struct knet_host *dst_host;
 	struct knet_link *dst_link;
 	int link_idx;
-	unsigned int min_mtu, have_mtu;
+	unsigned int have_mtu;
 	unsigned int lower_mtu;
 	int link_has_mtu;
 	int force_run = 0;
 
 	set_thread_status(knet_h, KNET_THREAD_PMTUD, KNET_THREAD_STARTED);
 
-	knet_h->data_mtu = KNET_PMTUD_MIN_MTU_V4 - KNET_HEADER_ALL_SIZE - knet_h->sec_header_size;
+	knet_h->data_mtu = calc_min_mtu(knet_h);
 
 	/* preparing pmtu buffer */
 	knet_h->pmtudbuf->kh_version = KNET_HEADER_VERSION;
@@ -514,7 +559,6 @@ void *_handle_pmtud_link_thread(void *data)
 		}
 
 		lower_mtu = KNET_PMTUD_SIZE_V4;
-		min_mtu = KNET_PMTUD_SIZE_V4 - KNET_HEADER_ALL_SIZE - knet_h->sec_header_size;
 		have_mtu = 0;
 
 		for (dst_host = knet_h->host_head; dst_host != NULL; dst_host = dst_host->next) {
@@ -523,20 +567,30 @@ void *_handle_pmtud_link_thread(void *data)
 
 				if ((dst_link->status.enabled != 1) ||
 				    (dst_link->status.connected != 1) ||
-				    (dst_host->link[link_idx].transport_type == KNET_TRANSPORT_LOOPBACK) ||
+				    (dst_host->link[link_idx].transport == KNET_TRANSPORT_LOOPBACK) ||
 				    (!dst_link->last_ping_size) ||
 				    ((dst_link->dynamic == KNET_LINK_DYNIP) &&
 				     (dst_link->status.dynconnected != 1)))
 					continue;
 
-				link_has_mtu = _handle_check_pmtud(knet_h, dst_host, dst_link, &min_mtu, force_run);
-				if (errno == EDEADLK) {
-					goto out_unlock;
-				}
-				if (link_has_mtu) {
-					have_mtu = 1;
-					if (min_mtu < lower_mtu) {
-						lower_mtu = min_mtu;
+				if (!knet_h->manual_mtu) {
+					link_has_mtu = _handle_check_pmtud(knet_h, dst_host, dst_link, force_run);
+					if (errno == EDEADLK) {
+						goto out_unlock;
+					}
+					if (link_has_mtu) {
+						have_mtu = 1;
+						if (dst_link->status.mtu < lower_mtu) {
+							lower_mtu = dst_link->status.mtu;
+						}
+					}
+				} else {
+					link_has_mtu = _calculate_manual_mtu(knet_h, dst_link);
+					if (link_has_mtu) {
+						have_mtu = 1;
+						if (dst_link->status.mtu < lower_mtu) {
+							lower_mtu = dst_link->status.mtu;
+						}
 					}
 				}
 			}
