@@ -4,7 +4,7 @@
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -20,6 +20,7 @@
 #include "crypto.h"
 #include "host.h"
 #include "links.h"
+#include "links_acl.h"
 #include "logging.h"
 #include "transports.h"
 #include "transport_common.h"
@@ -185,8 +186,10 @@ static int pckt_defrag(knet_handle_t knet_h, struct knet_header *inbuf, ssize_t
 		defrag_buf->frag_size = *len;
 	}
 
-	memmove(defrag_buf->buf + ((inbuf->khp_data_frag_seq - 1) * defrag_buf->frag_size),
-	       inbuf->khp_data_userdata, *len);
+	if (defrag_buf->frag_size) {
+		memmove(defrag_buf->buf + ((inbuf->khp_data_frag_seq - 1) * defrag_buf->frag_size),
+		       inbuf->khp_data_userdata, *len);
+	}
 
 	defrag_buf->frag_recv++;
 	defrag_buf->frag_map[inbuf->khp_data_frag_seq] = 1;
@@ -498,7 +501,6 @@ static void _parse_recv_from_links(knet_handle_t knet_h, int sockfd, const struc
 		} else { /* HOSTINFO */
 			knet_hostinfo = (struct knet_hostinfo *)inbuf->khp_data_userdata;
 			if (knet_hostinfo->khi_bcast == KNET_HOSTINFO_UCAST) {
-				bcast = 0;
 				knet_hostinfo->khi_dst_node_id = ntohs(knet_hostinfo->khi_dst_node_id);
 			}
 			if (!_seq_num_lookup(src_host, inbuf->khp_data_seq_num, 0, 0)) {
@@ -577,7 +579,7 @@ static void _parse_recv_from_links(knet_handle_t knet_h, int sockfd, const struc
 		}
 
 retry_pong:
-		if (transport_get_connection_oriented(knet_h, src_link->transport_type) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
+		if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
 			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
 				     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
 		} else {
@@ -585,7 +587,7 @@ retry_pong:
 		}
 		savederrno = errno;
 		if (len != outlen) {
-			err = transport_tx_sock_error(knet_h, src_link->transport_type, src_link->outsock, len, savederrno);
+			err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
 			switch(err) {
 				case -1: /* unrecoverable error */
 					log_debug(knet_h, KNET_SUB_RX,
@@ -615,37 +617,42 @@ retry_pong:
 		timespec_diff(recvtime,
 				src_link->status.pong_last, &latency_last);
 
-		src_link->status.latency =
-			((src_link->status.latency * src_link->latency_exp) +
-			((latency_last / 1000llu) *
-				(src_link->latency_fix - src_link->latency_exp))) /
-					src_link->latency_fix;
+		if ((latency_last / 1000llu) > src_link->pong_timeout) {
+			log_debug(knet_h, KNET_SUB_RX,
+				  "Incoming pong packet from host: %u link: %u has higher latency than pong_timeout. Discarding",
+				  src_host->host_id, src_link->link_id);
+		} else {
+			src_link->status.latency =
+				((src_link->status.latency * src_link->latency_exp) +
+				((latency_last / 1000llu) *
+					(src_link->latency_fix - src_link->latency_exp))) /
+						src_link->latency_fix;
 
-		if (src_link->status.latency < src_link->pong_timeout_adj) {
-			if (!src_link->status.connected) {
-				if (src_link->received_pong >= src_link->pong_count) {
-					log_info(knet_h, KNET_SUB_RX, "host: %u link: %u is up",
-						 src_host->host_id, src_link->link_id);
-					_link_updown(knet_h, src_host->host_id, src_link->link_id, src_link->status.enabled, 1);
-				} else {
-					src_link->received_pong++;
-					log_debug(knet_h, KNET_SUB_RX, "host: %u link: %u received pong: %u",
-						  src_host->host_id, src_link->link_id, src_link->received_pong);
+			if (src_link->status.latency < src_link->pong_timeout_adj) {
+				if (!src_link->status.connected) {
+					if (src_link->received_pong >= src_link->pong_count) {
+						log_info(knet_h, KNET_SUB_RX, "host: %u link: %u is up",
+							 src_host->host_id, src_link->link_id);
+						_link_updown(knet_h, src_host->host_id, src_link->link_id, src_link->status.enabled, 1);
+					} else {
+						src_link->received_pong++;
+						log_debug(knet_h, KNET_SUB_RX, "host: %u link: %u received pong: %u",
+							  src_host->host_id, src_link->link_id, src_link->received_pong);
+					}
 				}
 			}
+			/* Calculate latency stats */
+			if (src_link->status.latency > src_link->status.stats.latency_max) {
+				src_link->status.stats.latency_max = src_link->status.latency;
+			}
+			if (src_link->status.latency < src_link->status.stats.latency_min) {
+				src_link->status.stats.latency_min = src_link->status.latency;
+			}
+			src_link->status.stats.latency_ave =
+				(src_link->status.stats.latency_ave * src_link->status.stats.latency_samples +
+				 src_link->status.latency) / (src_link->status.stats.latency_samples+1);
+			src_link->status.stats.latency_samples++;
 		}
-		/* Calculate latency stats */
-		if (src_link->status.latency > src_link->status.stats.latency_max) {
-			src_link->status.stats.latency_max = src_link->status.latency;
-		}
-		if (src_link->status.latency < src_link->status.stats.latency_min) {
-			src_link->status.stats.latency_min = src_link->status.latency;
-		}
-		src_link->status.stats.latency_ave =
-			(src_link->status.stats.latency_ave * src_link->status.stats.latency_samples +
-			 src_link->status.latency) / (src_link->status.stats.latency_samples+1);
-		src_link->status.stats.latency_samples++;
-
 		break;
 	case KNET_HEADER_TYPE_PMTUD:
 		src_link->status.stats.rx_pmtu_packets++;
@@ -673,7 +680,7 @@ retry_pong:
 			goto out_pmtud;
 		}
 retry_pmtud:
-		if (transport_get_connection_oriented(knet_h, src_link->transport_type) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
+		if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
 			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
 				     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
 		} else {
@@ -681,7 +688,7 @@ retry_pmtud:
 		}
 		savederrno = errno;
 		if (len != outlen) {
-			err = transport_tx_sock_error(knet_h, src_link->transport_type, src_link->outsock, len, savederrno);
+			err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
 			switch(err) {
 				case -1: /* unrecoverable error */
 					log_debug(knet_h, KNET_SUB_RX,
@@ -802,6 +809,31 @@ static void _handle_recv_from_links(knet_handle_t knet_h, int sockfd, struct kne
 				goto exit_unlock;
 				break;
 			case 2: /* packet is data and should be parsed as such */
+				/*
+				 * processing incoming packets vs access lists
+				 */
+				if ((knet_h->use_access_lists) &&
+				    (transport_get_acl_type(knet_h, transport) == USE_GENERIC_ACL)) {
+					if (!check_validate(knet_h, sockfd, transport, msg[i].msg_hdr.msg_name)) {
+						char src_ipaddr[KNET_MAX_HOST_LEN];
+						char src_port[KNET_MAX_PORT_LEN];
+
+						memset(src_ipaddr, 0, KNET_MAX_HOST_LEN);
+						memset(src_port, 0, KNET_MAX_PORT_LEN);
+						if (knet_addrtostr(msg[i].msg_hdr.msg_name, sockaddr_len(msg[i].msg_hdr.msg_name),
+								   src_ipaddr, KNET_MAX_HOST_LEN,
+								   src_port, KNET_MAX_PORT_LEN) < 0) {
+
+							log_debug(knet_h, KNET_SUB_RX, "Packet rejected: unable to resolve host/port");
+						} else {
+							log_debug(knet_h, KNET_SUB_RX, "Packet rejected from %s/%s", src_ipaddr, src_port);
+						}
+						/*
+						 * continue processing the other packets
+						 */
+						continue;
+					}
+				}
 				_parse_recv_from_links(knet_h, sockfd, &msg[i]);
 				break;
 		}
@@ -839,6 +871,15 @@ void *_handle_recv_from_links_thread(void *data)
 	while (!shutdown_in_progress(knet_h)) {
 		nev = epoll_wait(knet_h->recv_from_links_epollfd, events, KNET_EPOLL_MAX_EVENTS, KNET_THREADS_TIMERES / 1000);
 
+		/*
+		 * the RX threads only need to notify that there has been at least
+		 * one successful run after queue flush has been requested.
+		 * See setfwd in handle.c
+		 */
+		if (get_thread_flush_queue(knet_h, KNET_THREAD_RX) == KNET_THREAD_QUEUE_FLUSH) {
+			set_thread_flush_queue(knet_h, KNET_THREAD_RX, KNET_THREAD_QUEUE_FLUSHED);
+		}
+
 		/*
 		 * we use timeout to detect if thread is shutting down
 		 */
