@@ -4,7 +4,7 @@
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -42,13 +42,12 @@ static int _dispatch_to_links(knet_handle_t knet_h, struct knet_host *dst_host,
 	struct knet_link *cur_link;
 
 	for (link_idx = 0; link_idx < dst_host->active_link_entries; link_idx++) {
-		sent_msgs = 0;
 		prev_sent = 0;
 		progress = 1;
 
 		cur_link = &dst_host->link[dst_host->active_links[link_idx]];
 
-		if (cur_link->transport_type == KNET_TRANSPORT_LOOPBACK) {
+		if (cur_link->transport == KNET_TRANSPORT_LOOPBACK) {
 			continue;
 		}
 
@@ -68,11 +67,11 @@ retry:
 		cur = &msg[prev_sent];
 
 		sent_msgs = _sendmmsg(dst_host->link[dst_host->active_links[link_idx]].outsock,
-				      transport_get_connection_oriented(knet_h, dst_host->link[dst_host->active_links[link_idx]].transport_type),
+				      transport_get_connection_oriented(knet_h, dst_host->link[dst_host->active_links[link_idx]].transport),
 				      &cur[0], msgs_to_send - prev_sent, MSG_DONTWAIT | MSG_NOSIGNAL);
 		savederrno = errno;
 
-		err = transport_tx_sock_error(knet_h, dst_host->link[dst_host->active_links[link_idx]].transport_type, dst_host->link[dst_host->active_links[link_idx]].outsock, sent_msgs, savederrno);
+		err = transport_tx_sock_error(knet_h, dst_host->link[dst_host->active_links[link_idx]].transport, dst_host->link[dst_host->active_links[link_idx]].outsock, sent_msgs, savederrno);
 		switch(err) {
 			case -1: /* unrecoverable error */
 				cur_link->status.stats.tx_data_errors++;
@@ -236,7 +235,6 @@ static int _parse_recv_from_sock(knet_handle_t knet_h, size_t inlen, int8_t chan
 						local_link->status.stats.tx_data_retries++;
 						buf += err;
 						buflen -= err;
-						usleep(KNET_THREADS_TIMERES / 16);
 						goto local_retry;
 					}
 					if (err == buflen) {
@@ -647,19 +645,25 @@ static void _handle_send_to_links(knet_handle_t knet_h, struct msghdr *msg, int
 		docallback = 1;
 		memset(&ev, 0, sizeof(struct epoll_event));
 
-		if (epoll_ctl(knet_h->send_to_links_epollfd,
-			      EPOLL_CTL_DEL, knet_h->sockfd[channel].sockfd[knet_h->sockfd[channel].is_created], &ev)) {
-			log_err(knet_h, KNET_SUB_TX, "Unable to del datafd %d from linkfd epoll pool: %s",
-				knet_h->sockfd[channel].sockfd[0], strerror(savederrno));
-		} else {
-			knet_h->sockfd[channel].has_error = 1;
+		if (channel != KNET_INTERNAL_DATA_CHANNEL) {
+			if (epoll_ctl(knet_h->send_to_links_epollfd,
+				      EPOLL_CTL_DEL, knet_h->sockfd[channel].sockfd[knet_h->sockfd[channel].is_created], &ev)) {
+				log_err(knet_h, KNET_SUB_TX, "Unable to del datafd %d from linkfd epoll pool: %s",
+					knet_h->sockfd[channel].sockfd[0], strerror(savederrno));
+			} else {
+				knet_h->sockfd[channel].has_error = 1;
+			}
 		}
+		/*
+		 * TODO: add error handling for KNET_INTERNAL_DATA_CHANNEL
+		 *       once we add support for internal knet communication
+		 */
 	} else {
 		knet_h->recv_from_sock_buf->kh_type = type;
 		_parse_recv_from_sock(knet_h, inlen, channel, 0);
 	}
 
-	if (docallback) {
+	if ((docallback) && (channel != KNET_INTERNAL_DATA_CHANNEL)) {
 		knet_h->sock_notify_fn(knet_h->sock_notify_fn_private_data,
 				       knet_h->sockfd[channel].sockfd[0],
 				       channel,
@@ -674,6 +678,7 @@ void *_handle_send_to_links_thread(void *data)
 	knet_handle_t knet_h = (knet_handle_t) data;
 	struct epoll_event events[KNET_EPOLL_MAX_EVENTS];
 	int i, nev, type;
+	int flush, flush_queue_limit;
 	int8_t channel;
 	struct iovec iov_in;
 	struct msghdr msg;
@@ -701,16 +706,49 @@ void *_handle_send_to_links_thread(void *data)
 		knet_h->send_to_links_buf[i]->kh_node = htons(knet_h->host_id);
 	}
 
+	flush_queue_limit = 0;
+
 	while (!shutdown_in_progress(knet_h)) {
 		nev = epoll_wait(knet_h->send_to_links_epollfd, events, KNET_EPOLL_MAX_EVENTS + 1, KNET_THREADS_TIMERES / 1000);
 
+		flush = get_thread_flush_queue(knet_h, KNET_THREAD_TX);
+
 		/*
 		 * we use timeout to detect if thread is shutting down
 		 */
 		if (nev == 0) {
+			/*
+			 * ideally we want to communicate that we are done flushing
+			 * the queue when we have an epoll timeout event
+			 */
+			if (flush == KNET_THREAD_QUEUE_FLUSH) {
+				set_thread_flush_queue(knet_h, KNET_THREAD_TX, KNET_THREAD_QUEUE_FLUSHED);
+				flush_queue_limit = 0;
+			}
 			continue;
 		}
 
+		/*
+		 * fall back in case the TX sockets will continue receive traffic
+		 * and we do not hit an epoll timeout.
+		 *
+		 * allow up to a 100 loops to flush queues, then we give up.
+		 * there might be more clean ways to do it by checking the buffer queue
+		 * on each socket, but we have tons of sockets and calculations can go wrong.
+		 * Also, why would you disable data forwarding and still send packets?
+		 */
+		if (flush == KNET_THREAD_QUEUE_FLUSH) {
+			if (flush_queue_limit >= 100) {
+				log_debug(knet_h, KNET_SUB_TX, "Timeout flushing the TX queue, expect packet loss");
+				set_thread_flush_queue(knet_h, KNET_THREAD_TX, KNET_THREAD_QUEUE_FLUSHED);
+				flush_queue_limit = 0;
+			} else {
+				flush_queue_limit++;
+			}
+		} else {
+			flush_queue_limit = 0;
+		}
+
 		if (pthread_rwlock_rdlock(&knet_h->global_rwlock) != 0) {
 			log_debug(knet_h, KNET_SUB_TX, "Unable to get read lock");
 			continue;
@@ -719,7 +757,7 @@ void *_handle_send_to_links_thread(void *data)
 		for (i = 0; i < nev; i++) {
 			if (events[i].data.fd == knet_h->hostsockfd[0]) {
 				type = KNET_HEADER_TYPE_HOST_INFO;
-				channel = -1;
+				channel = KNET_INTERNAL_DATA_CHANNEL;
 			} else {
 				type = KNET_HEADER_TYPE_DATA;
 				for (channel = 0; channel < KNET_DATAFD_MAX; channel++) {
@@ -740,6 +778,7 @@ void *_handle_send_to_links_thread(void *data)
 			_handle_send_to_links(knet_h, &msg, events[i].data.fd, channel, type);
 			pthread_mutex_unlock(&knet_h->tx_mutex);
 		}
+
 		pthread_rwlock_unlock(&knet_h->global_rwlock);
 	}
 
